<?php if (!defined('PmWiki')) exit();

/* creativecommons.php -- provides Creative Commons metadata for PmWiki pages
 * Copyright 2004, Evan Prodromou <evan@pigdog.org>.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


$CcNs = 'http://web.resource.org/cc/';
$CcPref = 'http://creativecommons.org/licenses/';

$CcLicenses = array('by', 'by-nd', 'by-nd-nc', 'by-nc', 
		    'by-nc-sa', 'by-sa', 'nd', 'nd-nc',
		    'nc', 'nc-sa', 'sa');

$KnownLicenses = array();

foreach ($CcLicenses as $license) {
    $KnownLicenses["http://creativecommons.org/licenses/{$license}/1.0/"] = explode('-', $license);
}

AddMetadata('cc', 'HandleCc');
		       
/* Handles outputting the Creative Commons RDF. */

function HandleCc($pagename) {
    
    global $WikiTitle, $RdfMimeType;
    global $AboutPage;
    global $CopyrightPage, $LicenseUrl, $CopyrightText;
    global $KnownLicenses, $CcNs;
    
    $page = RetrieveAuthPage($pagename,"read");
    if (!$page) Abort("Invalid page name");
    Lock(0);
    header("Content-Type: {$RdfMimeType}");
    
    if ($pagename == '') $pagename = $DefaultPage;
    $pageurl = FmtPageName('$PageUrl', $pagename);
    
    $last_author = @$page['author'];
    
    CcPrologue();
    CcSubPrologue('Work', $pageurl);    
    NsPageOrString('dc', 'publisher', $AboutPage, $WikiTitle);
    NsElement('dc', 'title', $pagename);
    NsElement('dc', 'date', strftime('%Y-%m-%d', $page['time']));
    NsPerson('dc', 'creator', $last_author);
    NsElement('dc', 'type', 'Text');
    NsElement('dc', 'format', 'text/html');
    NsElement('dc', 'identifier', $pageurl);

    $contributors = array();

    foreach ($page as $k=>$v) {
	if (preg_match('/^author\:\d+/', $k)) {
	    if (strcmp($v, $last_author) != 0 &&
		!in_array($v, $contributors)) {
		$contributors[] = $v;
	    }
	}
    }

    foreach ($contributors as $contrib) {
	NsPerson('dc', 'contributor', $contrib);
    }

    if (isset($CopyrightPage) && PageExists($CopyrightPage)) {
	NsPage('dc', 'rights', $CopyrightPage);
    } else if (isset($CopyrightText)) {
	NsElement('dc', 'rights', $CopyrightText);
    }

    if (isset($LicenseUrl)) {
	NsUrl('cc', 'license', $LicenseUrl);
    }

    CcSubEpilogue('Work');

    if (isset($LicenseUrl) && key_exists($LicenseUrl, $KnownLicenses)) {
	$elements = $KnownLicenses[$LicenseUrl];
	CcSubPrologue('License', $LicenseUrl);
	
	NsUrl('cc', 'permits', $CcNs . "Reproduction");
	NsUrl('cc', 'permits', $CcNs . "Distribution");
	NsUrl('cc', 'requires', $CcNs . "Notice");
	
	if (!in_array('nd', $elements)) {
	    NsUrl('cc', 'permits', $CcNs . "DerivativeWorks");
	}
	
	foreach ($elements as $el) {
	    switch ($el) {
	     case 'nc':	NsUrl('cc', 'prohibits', $CcNs . "CommercialUse"); break;
	     case 'by':	NsUrl('cc', 'requires', $CcNs . "Attribution"); break;
	     case 'sa':	NsUrl('cc', 'requires', $CcNs . "ShareAlike"); break;
	    }
	}
	
	CcSubEpilogue('License', $LicenseUrl);	
    }
    
    CcEpilogue();
}

/* The beginning boilerplate. */

function CcPrologue() {
    echo "<?xml version='1.0' encoding='iso-8859-1' ?>
	   
<rdf:RDF xmlns:cc=\"http://web.resource.org/cc/\"
         xmlns:dc=\"http://purl.org/dc/elements/1.1/\"
         xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\">
";
}  

function CcSubPrologue($type, $url) {
    echo "  <cc:{$type} rdf:about=\"{$url}\">\n";
}  

function CcSubEpilogue($type) {
    echo "  </cc:{$type}>\n";
}  

/* The ending boilerplate. */

function CcEpilogue() {
    echo "</rdf:RDF>\n";
}

?>
