<?php if (!defined('PmWiki')) exit();
/* dublincore.php -- provides Simple Dublin Core metadata for PmWiki pages
 * Copyright 2004, Evan Prodromou <evan@pigdog.org>.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

$DublinCoreVersion = $MetadataVersion;

AddMetadata('dc', 'HandleDc');

/* Handles outputting the Dublin Core RDF. */

function HandleDc($pagename) {
    
    global $WikiTitle, $RdfMimeType;
    global $AboutPage;
    global $CopyrightPage, $LicenseUrl, $CopyrightText;
    
    $page = RetrieveAuthPage($pagename,"read");
    if (!$page) Abort("Invalid page name");
    Lock(0);
    header("Content-Type: {$RdfMimeType}");
    
    if ($pagename == '') $pagename = $DefaultPage;
    $pageurl = FmtPageName('$PageUrl', $pagename);
    
    $last_author = @$page['author'];
    
    DcPrologue($pageurl);
    DcPageOrString('publisher', $AboutPage, $WikiTitle);
    DcElement('title', $pagename);
    DcElement('date', strftime('%Y-%m-%d', $page['time']));
    DcPerson('creator', $last_author);
    DcElement('type', 'Text');
    DcElement('format', 'text/html');
    DcElement('identifier', $pageurl);

    $contributors = array();

    foreach ($page as $k=>$v) {
	if (preg_match('/^author\:\d+/', $k)) {
	    if (strcmp($v, $last_author) != 0 &&
		!in_array($v, $contributors)) {
		$contributors[] = $v;
	    }
	}
    }

    foreach ($contributors as $contrib) {
	DcPerson('contributor', $contrib);
    }

    if (isset($CopyrightPage) && PageExists($CopyrightPage)) {
	DcPage('rights', $CopyrightPage);
    } else if (isset($LicenseUrl)) {
	DcUrl('rights', $LicenseUrl);
    } else if (isset($CopyrightText)) {
	DcElement('rights', $CopyrightText);
    }

    DcEpilogue();
}

/* The beginning boilerplate. */

function DcPrologue($pageurl) {
    echo "<?xml version='1.0' encoding='iso-8859-1' ?>
<!DOCTYPE rdf:RDF PUBLIC \"-//DUBLIN CORE//DCMES DTD 2002/07/31//EN\" \"http://dublincore.org/documents/2002/07/31/dcmes-xml/dcmes-xml-dtd.dtd\">
	   
<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\"
         xmlns:dc=\"http://purl.org/dc/elements/1.1/\">
  <rdf:Description rdf:about=\"$pageurl\">
";
}  

/* The ending boilerplate. */

function DcEpilogue() {
    echo "
  </rdf:Description>
</rdf:RDF>
";
}

/* Helpers, so we don't have to say 'dc' over and over again. */

function DcElement($name, $value) {
    NsElement('dc', $name, $value);
}

function DcPage($name, $page) {
    NsPage('dc', $name, $page);
}

function DcUrl($name, $url) {
    NsUrl('dc', $name, $url);
}

function DcPageOrString($name, $page, $string) {
    NsPageOrString('dc', $name, $page, $string);
}

function DcPerson($name, $person) {
    NsPerson('dc', $name, $person);
}

?>
